"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const AWS = require("aws-sdk");
const regexMatcher_1 = require("../lib/regexMatcher");
const chai_1 = require("chai");
const jsonata = require("jsonata");
exports.awsSdkStepRunners = ({ region, }) => [
    {
        willRun: regexMatcher_1.regexMatcher(/^I execute "([^"]+)" of the AWS ([^ ]+) SDK( with)?$/),
        run: async ([method, api, withArgs], step, runner) => {
            let argument;
            if (withArgs) {
                if (!step.interpolatedArgument) {
                    throw new Error('Must provide argument!');
                }
                argument = JSON.parse(step.interpolatedArgument);
            }
            await runner.progress('AWS-SDK', `${api}.${method}(${argument !== undefined ? JSON.stringify(argument) : ''})`);
            const a = new AWS[api]({
                region,
            });
            const res = await a[method](argument).promise();
            runner.store.awsSdk = {
                res,
            };
            return res;
        },
    },
    {
        willRun: regexMatcher_1.regexMatcher(/^(?:"([^"]+)" of )?the execution result should (equal|match) this JSON$/),
        run: async ([exp, equalOrMatch], step, runner) => {
            const { awsSdk } = runner.store;
            if (!step.interpolatedArgument) {
                throw new Error('Must provide argument!');
            }
            const j = JSON.parse(step.interpolatedArgument);
            const result = awsSdk.res;
            const fragment = exp ? jsonata(exp).evaluate(result) : result;
            if (equalOrMatch === 'match') {
                chai_1.expect(fragment).to.containSubset(j);
            }
            else {
                chai_1.expect(fragment).to.deep.equal(j);
            }
            return [fragment];
        },
    },
    {
        willRun: regexMatcher_1.regexMatcher(/^(?:"([^"]+)" of )?the execution result should equal ([0-9]+)$/),
        run: async ([exp, num], _, runner) => {
            const { awsSdk } = runner.store;
            const result = awsSdk.res;
            const fragment = exp ? jsonata(exp).evaluate(result) : result;
            chai_1.expect(fragment).to.equal(parseInt(num, 10));
            return [fragment];
        },
    },
    {
        willRun: regexMatcher_1.regexMatcher(/^I store "([^"]+)" of the execution result as "([^"]+)"$/),
        run: async ([expression, storeName], _, runner) => {
            const e = jsonata(expression);
            const { awsSdk } = runner.store;
            const result = e.evaluate(awsSdk.res);
            chai_1.expect(result).to.not.be.an('undefined');
            runner.store[storeName] = result;
            return result;
        },
    },
];
